/*
 * Copyright (c) 2008-2021, Hazelcast, Inc. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.hazelcast.config;

import com.hazelcast.internal.config.RingbufferStoreConfigReadOnly;
import com.hazelcast.test.HazelcastParallelClassRunner;
import com.hazelcast.test.annotation.ParallelJVMTest;
import com.hazelcast.test.annotation.QuickTest;
import org.junit.Test;
import org.junit.experimental.categories.Category;
import org.junit.runner.RunWith;

import java.util.Properties;

@RunWith(HazelcastParallelClassRunner.class)
@Category({QuickTest.class, ParallelJVMTest.class})
public class RingbufferStoreConfigReadOnlyTest {

    private RingbufferStoreConfig getReadOnlyConfig() {
        return new RingbufferStoreConfigReadOnly(new RingbufferStoreConfig());
    }

    @Test(expected = UnsupportedOperationException.class)
    public void setStoreImplementationOfReadOnlyRingbufferStoreConfigShouldFail() {
        getReadOnlyConfig().setStoreImplementation(null);
    }

    @Test(expected = UnsupportedOperationException.class)
    public void setEnabledOfReadOnlyRingbufferStoreConfigShouldFail() {
        getReadOnlyConfig().setEnabled(true);
    }

    @Test(expected = UnsupportedOperationException.class)
    public void setClassNameOfReadOnlyRingbufferStoreConfigShouldFail() {
        getReadOnlyConfig().setClassName("myRingbufferStore");
    }

    @Test(expected = UnsupportedOperationException.class)
    public void setPropertiesOfReadOnlyRingbufferStoreConfigShouldFail() {
        getReadOnlyConfig().setProperties(new Properties());
    }

    @Test(expected = UnsupportedOperationException.class)
    public void setPropertyOfReadOnlyRingbufferStoreConfigShouldFail() {
        getReadOnlyConfig().setProperty("name", "value");
    }

    @Test(expected = UnsupportedOperationException.class)
    public void setFactoryClassNameOfReadOnlyRingbufferStoreConfigShouldFail() {
        getReadOnlyConfig().setFactoryClassName("myRingbufferStoreFactory");
    }

    @Test(expected = UnsupportedOperationException.class)
    public void setFactoryImplementationOfReadOnlyRingbufferStoreConfigShouldFail() {
        getReadOnlyConfig().setFactoryImplementation(null);
    }
}
